@@ -1,12 +1,12 @@
 
-__version__ = '0.4.0'
+__version__ = '0.5.0'
 
 import argparse
 import logging
 import pathlib
 import sys
 
-from .asset import font, image, map, raw
+from .asset.builders import font, image, map, raw
 from .tool import cmake, flasher, metadata, packer, relocs, version
 
 
