@@ -2,10 +2,8 @@ import logging
 import pathlib
 import re
 
-import yaml
-
-from ..asset import font, image, map, raw
-from ..core.outputformat import CHeader, CSource, RawBinary
+from ..asset.builders import font, image, map, raw
+from ..asset.writer import AssetWriter
 from ..core.palette import Palette
 from ..core.tool import Tool
 
@@ -22,53 +20,10 @@ class Packer(Tool):
         self.parser.add_argument('--force', action='store_true', help='Force file overwrite')
 
         self.config = {}
-        self.assets = []
-        self.general_options = {}
-
-    def parse_config(self, config_file):
-        config = open(config_file).read()
-        config = yaml.safe_load(config)
-
-        self.config = config
-
-    def filelist_to_config(self, filelist, output):
-        config = {}
-
-        for file in filelist:
-            config[file] = {}
-
-        self.config = {f'{output}': config}
-
-    def get_general_options(self):
-        for key, value in self.config.items():
-            if key in ():
-                self.general_options[key] = value
-
-        for key, value in self.general_options.items():
-            self.config.items.pop(key)
+        self.targets = []
 
     def run(self, args):
-        self.working_path = pathlib.Path('.')
-
-        if args.config is not None:
-            if args.config.is_file():
-                self.working_path = args.config.parent
-            else:
-                logging.warning(f'Unable to find config at {args.config}')
-
-        if args.output is not None:
-            self.destination_path = args.output
-        else:
-            self.destination_path = self.working_path
-
-        if args.config is not None:
-            self.parse_config(args.config)
-            logging.info(f'Using config at {args.config}')
-
-        elif args.files is not None and args.output is not None:
-            self.filelist_to_config(args.files, args.output)
-
-        self.get_general_options()
+        self.setup_for_config(args.config, args.output, args.files)
 
         # Top level of our config is filegroups and general settings
         for target, options in self.config.items():
@@ -79,12 +34,12 @@ class Packer(Tool):
             target_options = {}
 
             for key, value in options.items():
-                if key in AssetTarget.supported_options:
+                if key in ('prefix', 'type'):
                     target_options[key] = value
 
             # Strip high-level options from the dict
             # Leaving just file source globs
-            for key, value in target_options.items():
+            for key in target_options:
                 options.pop(key)
 
             for file_glob, file_options in options.items():
@@ -114,7 +69,7 @@ class Packer(Tool):
                         AssetSource(input_files, file_options=file_opts, working_path=self.working_path)
                     )
 
-            self.assets.append(AssetTarget(
+            self.targets.append((
                 output_file,
                 asset_sources,
                 target_options
@@ -122,9 +77,13 @@ class Packer(Tool):
 
         self.destination_path.mkdir(parents=True, exist_ok=True)
 
-        for target in self.assets:
-            output = target.build()
-            self.output(output, self.destination_path / target.output_file.name, target.output_format, force=args.force)
+        for path, sources, options in self.targets:
+            aw = AssetWriter()
+            for source in sources:
+                for asset in source.build(output_file=path, prefix=options.get('prefix')):
+                    aw.add_asset(*asset)
+
+            aw.write(options.get('type'), self.destination_path / path.name, force=args.force)
 
 
 class AssetSource():
@@ -167,7 +126,7 @@ class AssetSource():
         logging.warning(f'Unable to guess type, assuming raw/binary {file}')
         return 'raw/binary'
 
-    def build(self, format, prefix=None, output_file=None):
+    def build(self, prefix=None, output_file=None):
         input_type, input_subtype = self.type.split('/')
         builder = self.asset_builders[input_type]()
 
@@ -186,8 +145,6 @@ class AssetSource():
                     if not pathlib.Path(self.builder_options[option_name]).is_absolute():
                         self.builder_options[option_name] = self.working_path / self.builder_options[option_name]
 
-        output = []
-
         for file in self.input_files:
             symbol_name = self.name
             if symbol_name is not None:
@@ -206,71 +163,10 @@ class AssetSource():
                 'input_file': file,
                 'output_file': output_file,
                 'input_type': input_subtype,
-                'output_format': format,
                 'symbol_name': symbol_name,
                 'working_path': self.working_path,
                 'prefix': prefix
             })
             builder.prepare_options(self.builder_options)
-            output.append(builder.build())
+            yield builder.build()
             logging.info(f' - {self.type} {file} -> {builder.symbol_name}')
-
-        return output
-
-
-class AssetTarget():
-    supported_options = ('prefix', 'type')
-    output_formats = {
-        '.hpp': CHeader,
-        '.cpp': CSource,
-        '.blit': RawBinary,
-        '.raw': RawBinary
-    }
-
-    def __init__(self, output_file, sources, target_options):
-        self.output_file = output_file
-        self.sources = sources
-        self.parse_target_options(target_options)
-
-    def parse_target_options(self, target_options):
-        self.prefix = target_options.get('prefix')
-        output_format = target_options.get('type')
-        if output_format is None:
-            output_format = self.guess_output_format(self.output_file)
-        else:
-            if type not in self.output_formats.values():
-                raise ValueError(f'Unknown asset output format {output_format}')
-        self.output_format = output_format
-
-    def build(self):
-        # TODO I hate this code, but because we're dealing with multiple input files
-        # the output can potentially be spread across multiple files with multiple snippets
-        # We need to gather each input files declarations/definitions (or otherwise) into
-        # the right output file.
-        output = []
-
-        # If the output_format deals with multiple files, switch to a dict and prep the files
-        if self.output_format.components is not None:
-            output = {}
-            for ext in self.output_format.components:
-                output[ext] = []
-
-        # Iterate through all the sources (input files) and build them
-        for source in self.sources:
-            data = source.build(format=self.output_format, output_file=self.output_file, prefix=self.prefix)
-            for snippet in data:
-                if self.output_format.components is not None:
-                    # If there are multiple output files, collect all the snippets into their respective outputs
-                    for ext, content in snippet.items():
-                        output[ext].append(content)
-                else:
-                    # If it's a single file format just append the snippet to the list
-                    output.append(snippet)
-
-        return output
-
-    def guess_output_format(self, file):
-        if file.suffix in self.output_formats:
-            return self.output_formats[file.suffix]
-        logging.warning(f'Unable to guess type of {file}, assuming raw/binary')
-        return RawBinary
